from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame

import easyglue
from test import EasyGlueTest
from test.test_utils import delete_all_s3_objects, list_all_objects


class WriterTest(EasyGlueTest):
    glue: GlueContext

    @classmethod
    def setUpClass(cls) -> None:
        super(WriterTest, cls).setUpClass()
        cls.dataset = cls.glue.create_dynamic_frame. \
            from_options(connection_type="s3",
                         connection_options={"paths": ["s3://bertolb/sampledata/mockaroo/json/"]},
                         format="json",
                         transformation_ctx="datasource0")

    @classmethod
    def tearDownClass(cls) -> None:
        delete_all_s3_objects("bertolb", "test/easyglue/outputs/")


def check_if_compressed(bucket: str, prefix: str) -> bool:
    objects = list_all_objects(bucket, prefix)
    first_object = objects[0]
    return first_object.endswith(".gz") or first_object.endswith(".bz2")


class TestEasyDynamicFrameWriter(WriterTest):
    glue: GlueContext
    dataset: DynamicFrame

    def test_format_option(self):
        output_path = "s3://bertolb/test/easyglue/outputs/format_option/"
        self.dataset.write.format_option("writeHeader", False).csv(output_path)
        data = self.glue.create_dynamic_frame. \
            from_options(connection_type="s3",
                         connection_options={"paths": [output_path]},
                         format="csv",
                         format_options={"withHeader": False})
        self.assertEqual(1000, data.count())

    def test_format_options(self):
        output_path = "s3://bertolb/test/easyglue/outputs/format_options/"
        self.dataset.write.format_options({"writeHeader": False}).csv(output_path)
        data = self.glue.create_dynamic_frame. \
            from_options(connection_type="s3",
                         connection_options={"paths": [output_path]},
                         format="csv",
                         format_options={"withHeader": False})
        self.assertEqual(1000, data.count())

    def test_connection_option(self):
        output_path = "s3://bertolb/test/easyglue/outputs/connection_option/"
        self.dataset.write.connection_option("compression", "gzip").json(output_path)
        compressed = check_if_compressed("bertolb", "test/easyglue/outputs/connection_option/")
        self.assertTrue(compressed)

    def test_connection_options(self):
        output_path = "s3://bertolb/test/easyglue/outputs/connection_options/"
        self.dataset.write.connection_options({"compression": "gzip"}).json(output_path)
        compressed = check_if_compressed("bertolb", "test/easyglue/outputs/connection_options/")
        self.assertTrue(compressed)

    # TODO test additional_options



DATABASE_NAME = "test"
TABLE_NAME = "easyglue_test"
TABLE_BUCKET = "bertolb"
TABLE_PREFIX = "test/easyglue/outputs/sampletable/"

table_def = {
    "Name": TABLE_NAME,
    "StorageDescriptor": {
        "Columns": [
            {
                "Name": "first_name",
                "Type": "string"
            },
            {
                "Name": "last_name",
                "Type": "string"
            },
            {
                "Name": "email",
                "Type": "string"
            },
            {
                "Name": "gender",
                "Type": "string"
            },
            {
                "Name": "ip_address",
                "Type": "string"
            },
            {
                "Name": "birth_date",
                "Type": "string"
            },
            {
                "Name": "employee_number",
                "Type": "string"
            },
            {
                "Name": "app_bundle_id",
                "Type": "string"
            },
            {
                "Name": "app_version",
                "Type": "string"
            },
            {
                "Name": "image_url",
                "Type": "string"
            },
            {
                "Name": "buzzword",
                "Type": "string"
            },
            {
                "Name": "car_brand",
                "Type": "string"
            },
            {
                "Name": "car_model",
                "Type": "string"
            },
            {
                "Name": "country",
                "Type": "string"
            },
            {
                "Name": "country_code",
                "Type": "string"
            },
            {
                "Name": "guid",
                "Type": "string"
            },
            {
                "Name": "isbn",
                "Type": "string"
            },
            {
                "Name": "mac_address",
                "Type": "string"
            },
            {
                "Name": "money",
                "Type": "string"
            },
            {
                "Name": "username",
                "Type": "string"
            },
            {
                "Name": "password",
                "Type": "string"
            },
            {
                "Name": "phone_number",
                "Type": "string"
            },
            {
                "Name": "postcode",
                "Type": "string"
            },
            {
                "Name": "sha256",
                "Type": "string"
            },
            {
                "Name": "time_12",
                "Type": "string"
            },
            {
                "Name": "time_24",
                "Type": "string"
            },
            {
                "Name": "timezone",
                "Type": "string"
            },
            {
                "Name": "website",
                "Type": "string"
            },
            {
                "Name": "domain_name",
                "Type": "string"
            },
            {
                "Name": "id",
                "Type": "string"
            },
            {
                "Name": "random_number",
                "Type": "string"
            },
            {
                "Name": "married",
                "Type": "string"
            },
            {
                "Name": "car_year",
                "Type": "string"
            },
            {
                "Name": "home_latitude",
                "Type": "string"
            },
            {
                "Name": "home_longitude",
                "Type": "string"
            }
        ],
        "Location": f"s3://{TABLE_BUCKET}/{TABLE_PREFIX}",
        "InputFormat": "org.apache.hadoop.mapred.TextInputFormat",
        "OutputFormat": "org.apache.hadoop.hive.ql.io.HiveIgnoreKeyTextOutputFormat",
        "Compressed": False,
        "NumberOfBuckets": -1,
        "SerdeInfo": {
            "SerializationLibrary": "org.openx.data.jsonserde.JsonSerDe",
            "Parameters": {
                "paths": "app_bundle_id,app_version,birth_date,buzzword,car_brand,car_model,car_year,country,country_code,domain_name,email,employee_number,first_name,gender,guid,home_latitude,home_longitude,id,image_url,ip_address,isbn,last_name,mac_address,married,money,password,phone_number,postcode,random_number,sha256,time_12,time_24,timezone,username,website"
            }
        }
    },
    "TableType": "EXTERNAL_TABLE",
}



import unittest

import easyglue
from easyglue.utils import get_connection_options_from_secret
from test.reader import ReaderTest

documentdb_connection_options = get_connection_options_from_secret("documentdb")
documentdb_connection_options.update({"ssl": "true",
                                      "ssl.domain_match": "false",
                                      "partitioner": "MongoSamplePartitioner",
                                      "partitionerOptions.partitionSizeMB": "10",
                                      "partitionerOptions.partitionKey": "_id"})


class TestReadOthers(ReaderTest):

    def test_dynamodb(self):
        data = self.glue.read.dynamodb("easyglue")
        self.assertEqual(4, data.count())

    def test_ddb(self):
        data = self.glue.read.ddb("easyglue")
        self.assertEqual(4, data.count())

    def test_documentdb(self):
        print(documentdb_connection_options)
        data = self.glue.read.connection_options(documentdb_connection_options).\
            documentdb(database="test", collection="collection")
        self.assertEqual(1, data.count())

    # TODO test mongo


if __name__ == '__main__':
    unittest.main()



import unittest

from awsglue.context import GlueContext

import easyglue
from test import EasyGlueTest


class ReaderTest(EasyGlueTest):
    glue: GlueContext

    @classmethod
    def setUpClass(cls) -> None:
        super(ReaderTest, cls).setUpClass()


class TestEasyDynamicFrameReader(EasyGlueTest):

    def test_format_option(self):
        data = self.glue.read.format_option("withHeader", True).csv("s3://bertolb/sampledata/mockaroo/csv/")
        self.assertEqual(1000, data.count())

    def test_format_options(self):
        data = self.glue.read.format_options({"withHeader": True}).csv("s3://bertolb/sampledata/mockaroo/csv/")
        self.assertEqual(1000, data.count())

    def test_connection_option(self):
        data = self.glue.read.connection_option("compression", "gzip")\
            .json("s3://bertolb/sampledata/mockaroo/json-gzip/")
        self.assertEqual(1000, data.count())

    def test_connection_options(self):
        data = self.glue.read.connection_options({"compression": "gzip"})\
            .json("s3://bertolb/sampledata/mockaroo/json-gzip/")
        self.assertEqual(1000, data.count())

    # TODO test additional_options


if __name__ == '__main__':
    unittest.main()



import unittest


class TestUtils(unittest.TestCase):

    def test_validate_qualified_name(self):
        from easyglue.utils import validate_qualified_name
        database, table = validate_qualified_name("default.legislators")
        self.assertEqual("default", database)
        self.assertEqual("legislators", table)

    def test_validate_qualified_name_error(self):
        from easyglue.utils import validate_qualified_name
        self.assertRaises(ValueError, validate_qualified_name, "legislators")


if __name__ == '__main__':
    unittest.main()



import unittest

from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame

import easyglue
from test.test_utils import ddb_client
from test.writer import WriterTest

DDB_TABLE_NAME = "easyglue-test"


def create_sample_ddb_table():
    delete_sample_ddb_table()
    _create_ddb_table(DDB_TABLE_NAME)


def delete_sample_ddb_table():
    if _ddb_table_exists(DDB_TABLE_NAME):
        _delete_ddb_table(DDB_TABLE_NAME)


def _ddb_table_exists(table_name: str):
    return table_name in ddb_client.list_tables().get('TableNames', [])


def _create_ddb_table(table_name: str):
    key_schema = [{'AttributeName': 'id', 'KeyType': 'HASH'}]
    attribute_definitions = [{'AttributeName': 'id', 'AttributeType': 'S'}]
    provisioned_throughput = {'ReadCapacityUnits': 50, 'WriteCapacityUnits': 50}

    ddb_client.create_table(TableName=table_name, KeySchema=key_schema, AttributeDefinitions=attribute_definitions,
                            ProvisionedThroughput=provisioned_throughput)
    waiter = ddb_client.get_waiter('table_exists')
    waiter.wait(TableName=table_name)


def _delete_ddb_table(table_name: str):
    """
    Deletes a DDB table by the given name, then waits until it's deleted
    :param table_name: Name of the table to delete
    :return: None
    """
    ddb_client.delete_table(TableName=table_name)
    waiter = ddb_client.get_waiter('table_not_exists')
    waiter.wait(TableName=table_name)


class TestOthers(WriterTest):
    _glue: GlueContext
    dataset: DynamicFrame

    @classmethod
    def setUpClass(cls) -> None:
        super(TestOthers, cls).setUpClass()
        create_sample_ddb_table()

    @classmethod
    def tearDownClass(cls) -> None:
        delete_sample_ddb_table()

    @unittest.skip("DDB tests take very long to complete, only enable when necessary")
    def test_dynamodb(self):
        self.dataset.write.dynamodb(table_name=DDB_TABLE_NAME)
        data = self.glue.create_dynamic_frame. \
            from_options(connection_type="dynamodb", connection_options={"dynamodb.input.tableName": DDB_TABLE_NAME})
        self.assertEqual(1000, data.count())

    @unittest.skip("DDB tests take very long to complete, only enable when necessary")
    def test_ddb(self):
        self.dataset.write.ddb(table_name=DDB_TABLE_NAME)
        data = self.glue.create_dynamic_frame. \
            from_options(connection_type="dynamodb", connection_options={"dynamodb.input.tableName": DDB_TABLE_NAME})
        self.assertEqual(1000, data.count())



import unittest

from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame

import easyglue
from test.writer import WriterTest


class TestS3Read(WriterTest):
    _glue: GlueContext
    dataset: DynamicFrame

    def test_no_params(self):
        self.assertRaises(ValueError, self.dataset.write.csv, "")

    def test_validate_mandatory_parameters(self):
        self.assertRaises(ValueError, self.dataset.write._validate_mandatory_parameters, '', '')
        self.assertRaises(ValueError, self.dataset.write._validate_mandatory_parameters, 'a', '')
        self.assertRaises(ValueError, self.dataset.write._validate_mandatory_parameters, '', 'a')

    def test_csv(self):
        output_path = "s3://bertolb/test/easyglue/outputs/csv/"
        self.dataset.write.csv(output_path)
        data = self.glue.create_dynamic_frame. \
            from_options(connection_type="s3",
                         connection_options={"paths": [output_path]},
                         format="csv",
                         format_options={"withHeader": True})
        self.assertEqual(1000, data.count())

    def test_json(self):
        output_path = "s3://bertolb/test/easyglue/outputs/json/"
        self.dataset.write.json(output_path)
        data = self.glue.create_dynamic_frame. \
            from_options(connection_type="s3",
                         connection_options={"paths": [output_path]},
                         format="json")
        self.assertEqual(1000, data.count())

    def test_avro(self):
        output_path = "s3://bertolb/test/easyglue/outputs/avro/"
        self.dataset.write.avro(output_path)
        data = self.glue.create_dynamic_frame. \
            from_options(connection_type="s3",
                         connection_options={"paths": [output_path]},
                         format="avro")
        self.assertEqual(1000, data.count())

    def test_orc(self):
        output_path = "s3://bertolb/test/easyglue/outputs/orc/"
        self.dataset.write.orc(output_path)
        data = self.glue.create_dynamic_frame. \
            from_options(connection_type="s3",
                         connection_options={"paths": [output_path]},
                         format="orc")
        self.assertEqual(1000, data.count())

    def test_parquet(self):
        output_path = "s3://bertolb/test/easyglue/outputs/parquet/"
        self.dataset.write.parquet(output_path)
        data = self.glue.create_dynamic_frame. \
            from_options(connection_type="s3",
                         connection_options={"paths": [output_path]},
                         format="parquet")
        self.assertEqual(1000, data.count())

    def test_glueparquet(self):
        output_path = "s3://bertolb/test/easyglue/outputs/glueparquet/"
        self.dataset.write.glueparquet(output_path)
        data = self.glue.create_dynamic_frame. \
            from_options(connection_type="s3",
                         connection_options={"paths": [output_path]},
                         format="parquet")
        self.assertEqual(1000, data.count())


if __name__ == '__main__':
    unittest.main()



from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame

import easyglue
from test.writer import WriterTest
from test.test_utils import delete_all_s3_objects, glue_client
from test.test_utils.resources import DATABASE_NAME, TABLE_NAME, TABLE_BUCKET, TABLE_PREFIX, table_def


def create_sample_table():
    delete_sample_table()
    _create_table(DATABASE_NAME, table_def)


def delete_sample_table():
    if _table_exists(DATABASE_NAME, TABLE_NAME):
        _delete_table(DATABASE_NAME, TABLE_NAME)


def _table_exists(database_name: str, table_name: str):
    tables = glue_client.get_tables(DatabaseName=database_name).get('TableList', [])
    table_names = list(map(lambda t: t['Name'], tables))
    return table_name in table_names


def _create_table(database_name: str, table_input: dict):
    glue_client.create_table(DatabaseName=database_name, TableInput=table_input)


def _delete_table(database_name: str, table_name: str):
    glue_client.delete_table(DatabaseName=database_name, Name=table_name)


class TestCatalog(WriterTest):
    _glue: GlueContext
    dataset: DynamicFrame

    @classmethod
    def tearDown(cls) -> None:
        delete_all_s3_objects(TABLE_BUCKET, TABLE_PREFIX)

    @classmethod
    def setUpClass(cls) -> None:
        super(TestCatalog, cls).setUpClass()
        create_sample_table()

    @classmethod
    def tearDownClass(cls) -> None:
        delete_sample_table()

    def test_catalog(self):
        self.dataset.write.catalog(database=DATABASE_NAME, table=TABLE_NAME)
        data = self.glue.create_dynamic_frame. \
            from_catalog(database=DATABASE_NAME,
                         table_name=TABLE_NAME)
        self.assertEqual(1000, data.count())

    def test_table(self):
        self.dataset.write.table(f"{DATABASE_NAME}.{TABLE_NAME}")
        data = self.glue.create_dynamic_frame. \
            from_catalog(database=DATABASE_NAME,
                         table_name=TABLE_NAME)
        self.assertEqual(1000, data.count())



import unittest

import easyglue
from easyglue.utils import get_connection_options_from_secret
from test.reader import ReaderTest

SECRET_NAME = "mysql"
TABLE_NAME = "easyglue"


class TestJDBC(ReaderTest):

    @classmethod
    def setUpClass(cls) -> None:
        super(TestJDBC, cls).setUpClass()
        cls.jdbc_options = get_connection_options_from_secret(SECRET_NAME, TABLE_NAME)

    def test_jdbc(self):
        data = self.glue.read.jdbc(self.jdbc_options)
        self.assertEqual(1000, data.count())

    def test_secret(self):
        data = self.glue.read.secret(table=TABLE_NAME, secret=SECRET_NAME)
        self.assertEqual(1000, data.count())


if __name__ == '__main__':
    unittest.main()



import unittest

from pyspark.context import SparkContext
from awsglue.context import GlueContext


class EasyGlueTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls) -> None:
        cls.glue = GlueContext(SparkContext.getOrCreate())



import unittest

import easyglue
from test.reader import ReaderTest


class TestCatalogRead(ReaderTest):

    def test_table(self):
        data = self.glue.read.table("sampledata.mockaroo_csv")
        self.assertEqual(1000, data.count())

    def test_catalog(self):
        data = self.glue.read.catalog("sampledata", "mockaroo_csv")
        self.assertEqual(1000, data.count())


if __name__ == '__main__':
    unittest.main()



from easyglue.utils import writer_method

from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame


class S3Mixin:
    glue: GlueContext
    dyf: DynamicFrame

    data_format: str
    connection_options_dict: dict
    format_options_dict: dict

    @writer_method
    def _write_to_s3(self, path: str, data_format: str, transformation_ctx: str = ""):
        """
        Writes a dataset to S3 by calling write_dynamic_frame.from_options with the right configuration
        :param path: S3 path to write to
        :param data_format: Data format to use when writing
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        self._validate_mandatory_parameters(path, data_format)
        self.glue.write_dynamic_frame.from_options(frame=self.dyf,
                                                   connection_type="s3",
                                                   connection_options=self.connection_options_dict,
                                                   format=self.data_format,
                                                   format_options=self.format_options_dict,
                                                   transformation_ctx=transformation_ctx)

    def _validate_mandatory_parameters(self, path: str, data_format: str):
        """
        Validates the existence of the 'path' and 'data_format' parameters, otherwise raises exception
        :param path: S3 path to write to
        :param data_format: Data format to use when writing
        :return: None
        """
        self.connection_options_dict['path'] = path
        self.data_format = data_format
        if not self.connection_options_dict.get('path'):
            raise ValueError(f'S3 path was not provided')
        if not self.data_format:
            raise ValueError(f'Data format was not provided')

    def csv(self, path: str, transformation_ctx: str = ""):
        """
        Writes a CSV dataset by calling the _write_to_s3 method with the right configuration
        :param path: S3 path to write to
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        self._write_to_s3(path=path, data_format="csv", transformation_ctx=transformation_ctx)

    def json(self, path: str, transformation_ctx: str = ""):
        """
        Writes a JSON dataset by calling the _write_to_s3 method with the right configuration
        :param path: S3 path to write to
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        self._write_to_s3(path=path, data_format="json", transformation_ctx=transformation_ctx)

    def avro(self, path: str, transformation_ctx: str = ""):
        """
        Writes a Avro dataset by calling the _write_to_s3 method with the right configuration
        :param path: S3 path to write to
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        self._write_to_s3(path=path, data_format="avro", transformation_ctx=transformation_ctx)

    def orc(self, path: str, transformation_ctx: str = ""):
        """
        Writes a ORC dataset by calling the _write_to_s3 method with the right configuration
        :param path: S3 path to write to
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        self._write_to_s3(path=path, data_format="orc", transformation_ctx=transformation_ctx)

    def parquet(self, path: str, transformation_ctx: str = ""):
        """
        Writes a Parquet dataset by calling the _write_to_s3 method with the right configuration
        :param path: S3 path to write to
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        self._write_to_s3(path=path, data_format="parquet", transformation_ctx=transformation_ctx)

    def glueparquet(self, path: str, transformation_ctx: str = ""):
        """
        Writes a Parquet dataset using GlueParquet by calling the _write_to_s3 method with the right configuration
        :param path: S3 path to write to
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        self._write_to_s3(path=path, data_format="glueparquet", transformation_ctx=transformation_ctx)



from easyglue.reader import EasyDynamicFrameReader
from easyglue.writer import EasyDynamicFrameWriter

from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame


@property
def read(self):  # Have to add self since this will become a method
    return EasyDynamicFrameReader(glue_context=self)


@property
def write(self):
    return EasyDynamicFrameWriter(glue_context=self.glue_ctx, dynamicframe=self)


setattr(GlueContext, 'read', read)
setattr(DynamicFrame, 'write', write)



import json
import re
import requests

import boto3

AWS_METADATA_ENDPOINT = 'http://169.254.169.254/latest/dynamic/instance-identity/document'


def get_current_region() -> str:
    response = requests.get(AWS_METADATA_ENDPOINT)
    return response.json().get('region')


secrets_client = boto3.client('secretsmanager', region_name=get_current_region())


def reader_method(func):
    """
    Decorator method, designed to be used in all methods that read a dataset and return it as a DynamicFrame. The
    decorator will call the wrapped method, then will clear all the options dictionaries so that options from a read
    don't reappear in the next read.
    :param func: Reader function
    :return: Result of the reader function
    """

    def wrapper(self, *args, **kwargs):
        result = func(self, *args, **kwargs)
        self.connection_options_dict.clear()
        self.format_options_dict.clear()
        self.additional_options_dict.clear()
        return result

    return wrapper


def writer_method(func):
    """
    Decorator method, designed to be used in all methods that write a dataset. The decorator will call the wrapped
    method, then will clear all the options dictionaries so that options from a write don't reappear in the next write.
    :param func: Writer function
    :return: Result of the writer function
    """
    def wrapper(self, *args, **kwargs):
        result = func(self, *args, **kwargs)
        self.connection_options_dict.clear()
        self.format_options_dict.clear()
        self.additional_options_dict.clear()
        return result

    return wrapper


QUALIFIED_NAME_MATCH_REGEX = "[a-zA-Z0-9_]+\\.[a-zA-Z0-9_]+"
QUALIFIED_NAME_MATCH_CAPTURE = "[a-zA-Z0-9_]+"


def validate_qualified_name(qualified_name: str) -> tuple:
    """
    Validates that the provided qualified name is in the form of "database.table" and if so, returns each part
    :param qualified_name: Qualified name of the table
    :return: Database name, table name
    """
    if not re.match(QUALIFIED_NAME_MATCH_REGEX, qualified_name):
        raise ValueError('Provided table name is not in the form of "database.table"')
    else:
        matches = re.findall(QUALIFIED_NAME_MATCH_CAPTURE, qualified_name)
        return matches[0], matches[1]


SUPPORTED_JDBC_ENGINES = ["mysql", "aurora", "redshift", "oracle", "postgresql"]
SUPPORTED_MONGO_ENGINES = ["mongo"]


def get_connection_options_from_secret(secret_id: str, table_name: str = None) -> dict:
    """
    Reads and parses the arguments of an AWS Secrets Manager secret into the connection options needed by Glue to
    connect to a JDBC or MongoDB/DocumentDB source.
    :param secret_id: Name or ARN of the AWS Secrets Manager secret to get the connection parameters from
    :param table_name: (Optional) Name of the table to read from
    :return: Dictionary of connection options
    """
    value = secrets_client.get_secret_value(SecretId=secret_id)
    secret = json.loads(value.get('SecretString', {}))
    engine = secret.get('engine')
    if engine in SUPPORTED_JDBC_ENGINES:
        return _generate_conn_opts_jdbc(secret, table_name)
    elif engine in SUPPORTED_MONGO_ENGINES:
        return _generate_conn_opts_mongo(secret)
    else:
        raise ValueError(f"The engine parameter of the specified Secrets Manager secret either does not exist or is "
                         f"not supported. Supported values are "
                         f"{', '.join(SUPPORTED_JDBC_ENGINES + SUPPORTED_MONGO_ENGINES)}")


def _generate_conn_opts_jdbc(secret: dict, table_name: str = None) -> dict:
    """
    Parses the provided secret's parameters into the ones expected by JDBC Glue connection options
    :param secret: AWS Secrets Manager secret values in dictionary form
    :param table_name: (Optional) Name of the table to read from
    :return: Dictionary of connection options
    """
    return {
        "url": f"jdbc:{secret.get('engine')}://{secret.get('host')}:{secret.get('port')}/{secret.get('dbname')}",
        "dbtable": table_name,
        "user": secret.get('username'),
        "password": secret.get('password')
    }


def _generate_conn_opts_mongo(secret: dict) -> dict:
    """
    Parses the provided secret's parameters into the ones expected by MongoDB/DocumentDB Glue connection options
    :param secret: AWS Secrets Manager secret values in dictionary form
    :return: Dictionary of connection options
    """
    return {
        "uri": f"mongodb://{secret.get('host')}:{secret.get('port')}",
        "username": secret.get('username'),
        "password": secret.get('password')
    }



from typing import Any

from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame

from easyglue.utils import reader_method


class OthersMixin:
    glue_context: GlueContext
    connection_options_dict: dict

    @reader_method
    def _read_from_others(self, connection_type: str, table_name: str = None, database_name: str = None,
                          transformation_ctx: str = "", **kwargs: Any) -> DynamicFrame:
        """
        Reads from NoSQL sources such as DynamoDB or MongoDB by calling create_dynamic_frame.from_options with the
        right configuration options
        :param connection_type: Connection type, can be "dynamodb", "mongodb" or "documentdb"
        :param table_name: Name of the table/collection to read from
        :param database_name: Name of the database (if any) holding the table
        :param transformation_ctx: Glue transformation context
        :param kwargs: Keyword arguments
        :return: DynamicFrame representing the dataset
        """
        if connection_type == "dynamodb" and table_name:
            self.connection_options_dict['dynamodb.input.tableName'] = table_name
        if connection_type in ["mongodb", "documentdb"] and database_name and table_name:
            self.connection_options_dict['database'] = database_name
            self.connection_options_dict['collection'] = table_name

        return self.glue_context.create_dynamic_frame_from_options(connection_type=connection_type,
                                                                   connection_options=self.connection_options_dict,
                                                                   transformation_ctx=transformation_ctx,
                                                                   kwargs=kwargs
                                                                   )

    def dynamodb(self, table_name: str = None, transformation_ctx: str = None, **kwargs: Any) -> DynamicFrame:
        """
        Reads a DynamoDB dataset by calling _read_from_others with the right configuration
        :param table_name: Name of the DynamoDB table
        :param transformation_ctx: Glue transformation context
        :param kwargs: Keyword arguments
        :return: DynamicFrame representing the dataset
        """
        return self._read_from_others(connection_type="dynamodb", table_name=table_name,
                                      transformation_ctx=transformation_ctx, kwargs=kwargs)

    ddb = dynamodb  # Same method with an easier, shorter syntax

    def mongodb(self, database: str = None, collection: str = None, transformation_ctx: str = "",
                **kwargs: Any) -> DynamicFrame:
        """
        Reads a MongoDB dataset with calling _read_from_others with the right configuration
        :param database: Name of the MongoDB database
        :param collection: Name of the MongoDB collection
        :param transformation_ctx: Glue transformation context
        :param kwargs: Keyword arguments
        :return: DynamicFrame representing the dataset
        """
        return self._read_from_others(connection_type="mongodb", table_name=collection, database_name=database,
                                      transformation_ctx=transformation_ctx, kwargs=kwargs)

    def documentdb(self, database: str = None, collection: str = None, transformation_ctx: str = "",
                   **kwargs: Any) -> DynamicFrame:
        """
        Reads a DocumentDB dataset with calling _read_from_others with the right configuration
        :param database: Name of the DocumentDB database
        :param collection: Name of the DocumentDB collection
        :param transformation_ctx: Glue transformation context
        :param kwargs: Keyword arguments
        :return: DynamicFrame representing the dataset
        """
        return self._read_from_others(connection_type="documentdb", table_name=collection, database_name=database,
                                      transformation_ctx=transformation_ctx, kwargs=kwargs)



from awsglue.context import GlueContext

from easyglue.utils import reader_method, get_connection_options_from_secret


class JDBCMixin:
    glue_context: GlueContext
    connection_options_dict: dict

    @reader_method
    def _read_from_jdbc(self, connection_type: str, transformation_ctx: str = "", push_down_predicate: str = ""):
        """
        Reads a dataset from a JDBC source by calling create_dynamic_frame.from_options with the right configuration
        :param connection_type: Connection type (database engine)
        :param transformation_ctx: Glue transformation context
        :param push_down_predicate: SQL pushdown predicate
        :return: DynamicFrame representing the JDBC table
        """
        return self.glue_context.create_dynamic_frame.from_options(connection_type=connection_type,
                                                                   connection_options=self.connection_options_dict,
                                                                   transformation_ctx=transformation_ctx,
                                                                   push_down_predicate=push_down_predicate)

    def jdbc(self, connection_options: dict = None, transformation_ctx: str = "", push_down_predicate: str = ""):
        """
        Reads a dataset from a JDBC source by calling _read_from_jdbc with the right configuration
        :param connection_options: Connection options dictionary specifying JDBC parameters
        :param transformation_ctx: Glue transformation context
        :param push_down_predicate: Push down predicate to be applied
        :return: DynamicFrame representing the JDBC table
        """
        if connection_options:
            self.connection_options_dict = connection_options
        url = self.connection_options_dict.get('url')
        db_type = url.split(':')[1]

        return self._read_from_jdbc(connection_type=db_type, transformation_ctx=transformation_ctx,
                                    push_down_predicate=push_down_predicate)

    def secret(self, table: str, secret: str, transformation_ctx: str = "", push_down_predicate: str = ""):
        """
        Reads a dataset from a JDBC source by getting the connection parameters from an AWS Secrets Manager secret
        :param table: Name of the JDBC table to read from
        :param secret: AWS Secrets Manager secret
        :param transformation_ctx: Glue transformation context
        :param push_down_predicate: SQL pushdown predicate
        :return: DynamicFrame representing the JDBC table
        """
        self.connection_options_dict = get_connection_options_from_secret(secret, table)
        url = self.connection_options_dict.get('url')
        db_type = url.split(':')[1]
        return self._read_from_jdbc(connection_type=db_type, transformation_ctx=transformation_ctx,
                                    push_down_predicate=push_down_predicate)



import boto3

s3_client = boto3.client('s3', region_name="eu-west-1")
ddb_client = boto3.client('dynamodb', region_name="eu-west-1")
glue_client = boto3.client('glue', region_name="eu-west-1")
secrets_client = boto3.client('secretsmanager', region_name="eu-west-1")


def delete_all_s3_objects(bucket: str, prefix: str):
    """
    Deletes all s3 objects in the given bucket and prefix
    :param bucket: S3 bucket
    :param prefix: S3 prefix
    :return: None
    """
    objects = list_all_objects(bucket, prefix)
    if objects:
        mapped_objects = list(map(lambda o: {'Key': o}, objects))
        s3_client.delete_objects(Bucket="bertolb", Delete={'Objects': mapped_objects})


def list_all_objects(bucket_name: str, prefix: str) -> list:
    """
    Lists all s3 objects in the given bucket and prefix
    :param bucket_name: S3 bucket
    :param prefix: S3 prefix
    :return: List of object keys
    """
    kwargs = {'Bucket': bucket_name, 'Prefix': prefix}
    object_list = []

    while True:
        resp = s3_client.list_objects_v2(**kwargs)
        object_list += list(map(lambda x: x.get('Key', ''), resp.get('Contents', [])))
        try:
            kwargs['ContinuationToken'] = resp['NextContinuationToken']
        except KeyError:
            break

    return object_list



from typing import Any

from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame

from easyglue.utils import writer_method


class JDBCMixin:
    glue: GlueContext
    dyf: DynamicFrame

    connection_options_dict: dict

    @writer_method
    def jdbc(self, connection_options: dict = None, transformation_ctx: str = "", **kwargs: Any):
        """
        Writes a dataset to a JDBC source by calling write_from_options with the right configuration
        :param connection_options: Connection options dictionary specifying JDBC parameters
        :param transformation_ctx: Glue transformation context
        :param kwargs: Keyword arguments
        :return: None
        """
        if connection_options:
            self.connection_options_dict = connection_options
        url = self.connection_options_dict.get('url')
        db_type = url.split(':')[1]
        self.glue.write_from_options(frame_or_dfc=self.dyf,
                                     connection_type=db_type,
                                     connection_options=self.connection_options_dict,
                                     transformation_ctx=transformation_ctx,
                                     kwargs=kwargs)



from typing import Any

from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame

from easyglue.utils import writer_method, validate_qualified_name


class CatalogMixin:
    glue: GlueContext
    dyf: DynamicFrame

    additional_options_dict: dict

    @writer_method
    def _write_to_catalog(self, database: str, table: str, redshift_tmp_dir: str = "", transformation_ctx: str = "",
                          catalog_id: int = None, **kwargs: Any):
        """
        Writes the DynamicFrame to catalog by invoking write_dynamic_frame.from_catalog with the right options
        :param database: Data catalog database to write to
        :param table: Data catalog table to write to
        :param redshift_tmp_dir: Redshift temporary path
        :param transformation_ctx: Glue transformation context
        :param catalog_id: Glue Data Catalog ID
        :param kwargs: Keyword arguments
        :return: None
        """
        self.glue.write_dynamic_frame.from_catalog(frame=self.dyf,
                                                   database=database,
                                                   table_name=table,
                                                   redshift_tmp_dir=redshift_tmp_dir,
                                                   transformation_ctx=transformation_ctx,
                                                   additional_options=self.additional_options_dict,
                                                   catalog_id=catalog_id,
                                                   kwargs=kwargs)

    def catalog(self, database: str, table: str, redshift_tmp_dir: str = "", transformation_ctx: str = "",
                catalog_id: int = None, **kwargs: Any):
        """
        Writes the DynamicFrame to catalog by invoking _write_to_catalog
        :param database: Data catalog database to write to
        :param table: Data catalog table to write to
        :param redshift_tmp_dir: Redshift temporary path
        :param transformation_ctx: Glue transformation context
        :param catalog_id: Glue Data Catalog ID
        :param kwargs: Keyword arguments
        :return: None
        """
        self._write_to_catalog(database=database, table=table, redshift_tmp_dir=redshift_tmp_dir,
                               transformation_ctx=transformation_ctx, catalog_id=catalog_id, kwargs=kwargs)

    def table(self, qualified_name: str, redshift_tmp_dir: str = "", transformation_ctx: str = "",
              catalog_id: int = None, **kwargs: Any):
        """
        Writes the DynamicFrame to catalog by invoking _write_to_catalog
        :param qualified_name: Qualified name (database.table) of the Data Catalog table to write to
        :param redshift_tmp_dir: Redshift temporary path
        :param transformation_ctx: Glue transformation context
        :param catalog_id: Glue Data Catalog ID
        :param kwargs: Keyword arguments
        :return: None
        """
        database_name, table_name = validate_qualified_name(qualified_name)
        self._write_to_catalog(database=database_name, table=table_name, redshift_tmp_dir=redshift_tmp_dir,
                               transformation_ctx=transformation_ctx, catalog_id=catalog_id, kwargs=kwargs)



from awsglue.context import GlueContext
from awsglue.dynamicframe import DynamicFrame

from easyglue.utils import writer_method


class OthersMixin:
    glue: GlueContext
    dyf: DynamicFrame

    connection_options_dict: dict

    @writer_method
    def _write_to_others(self, connection_type: str, table_name: str = None, database_name: str = None,
                         transformation_ctx: str = ""):
        """
        Internal method to write to DDB tables
        :param table_name: Name of the DynamoDB table
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        if connection_type == "dynamodb" and table_name:
            self.connection_options_dict['dynamodb.output.tableName'] = table_name
        if connection_type in ["mongodb", "documentdb"] and database_name and table_name:
            self.connection_options_dict['database'] = database_name
            self.connection_options_dict['collection'] = table_name
        self.glue.write_dynamic_frame.from_options(frame=self.dyf,
                                                   connection_type=connection_type,
                                                   connection_options=self.connection_options_dict,
                                                   transformation_ctx=transformation_ctx
                                                   )

    def dynamodb(self, table_name: str, transformation_ctx: str = ""):
        """
        Write a DynamoDB dataset by calling _write_to_others with the right configuration
        :param table_name: Name of the DynamoDB table
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        self._write_to_others(connection_type="dynamodb", table_name=table_name, transformation_ctx=transformation_ctx)

    ddb = dynamodb

    def mongodb(self, database: str = None, collection: str = None, transformation_ctx: str = ""):
        """
        Write a MongoDB dataset by calling _write_to_others with the right configuration
        :param database: Name of the database to write to
        :param collection: Name of the collection to write to
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        self._write_to_others(connection_type="mongodb", table_name=collection, database_name=database,
                              transformation_ctx=transformation_ctx)

    def documentdb(self, database: str = None, collection: str = None, transformation_ctx: str = ""):
        """
        Write a DocumentDB dataset by calling _write_to_others with the right configuration
        :param database: Name of the database to write to
        :param collection: Name of the collection to write to
        :param transformation_ctx: Glue transformation context
        :return: None
        """
        self._write_to_others(connection_type="documentdb", table_name=collection, database_name=database,
                              transformation_ctx=transformation_ctx)
